﻿namespace ElectricLocomotive;

public class LocosGenericCollection <T, U>  where T : DrawingLocomotiv where U : IMoveableObject
{
    private readonly int _pictureWidth;
    private readonly int _pictureHeight;
    private readonly int _placeSizeWidth = 170;
    private readonly int _placeSizeHeight = 110;
    private readonly SetGeneric<T> _collection;
    public IEnumerable<T?> GetLocos => _collection.GetElectricLocos();
    public LocosGenericCollection(int picWidth, int picHeight)
    {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = new SetGeneric<T>(width * height);
    }
    public static bool operator +(LocosGenericCollection<T, U> collect, T?
    obj)
    {
        if (obj == null) return false;

        return collect?._collection.Insert(obj) ?? false;
    }

    public static T? operator -(LocosGenericCollection<T, U> collect, int
    pos)
    {   
        T? obj = collect._collection[pos];
        collect._collection.Remove(pos);
        return obj;
    }
    public U? GetU(int pos)
    {
        return (U?)_collection[pos]?.GetMoveableObject;
    }
    public Bitmap ShowLocos()
    {
        Bitmap bmp = new(_pictureWidth, _pictureHeight);
        Graphics gr = Graphics.FromImage(bmp);
        DrawBackground(gr);
        DrawObjects(gr);
        return bmp;
    }
    private void DrawBackground(Graphics g)
    {
        Pen pen = new(Color.Black, 3);
        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight +
            1; ++j)
            {
                g.DrawLine(pen, i * _placeSizeWidth, j *
                _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                _placeSizeHeight);
            }
            g.DrawLine(pen, i * _placeSizeWidth, 0, i *
            _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
        }
    }
    private void DrawObjects(Graphics g)
    {
        int i = 0;
        foreach(var electricLoco in _collection.GetElectricLocos())
        {
            if (electricLoco != null)
            {
                int inRow = _pictureWidth / _placeSizeWidth;
                electricLoco.SetPosition(i % inRow * _placeSizeWidth, _pictureHeight - _pictureHeight % _placeSizeHeight - (i / inRow + 1) * _placeSizeHeight);
                electricLoco.DrawLoco(g);
            }
            i++;
        }
    }

}